#!/usr/bin/python
#Last-modified: 29 Sep 2013 10:11:26 PM

""" Module/Scripts Description

Copyright (c) 2008 Yunfei Wang <tszn1984@gmail.com>

This code is free software; you can redistribute it and/or modify it
under the terms of the BSD License (see the file COPYING included with
the distribution).

@status:  experimental
@version: 1.0.0
@author:  Yunfei Wang
@contact: tszn1984@gmail.com
"""

# ------------------------------------
# python modules
# ------------------------------------

import sys
import string
from ngslib import IO,GeneBedList,BedList,Utils

# ------------------------------------
# constants
# ------------------------------------

# ------------------------------------
# Misc functions
# ------------------------------------

# ------------------------------------
# Classes
# ------------------------------------

# ------------------------------------
# Main
# ------------------------------------

if __name__=="__main__":
    if len(sys.argv)==1:
        print "Usage: "+sys.argv[0]+" annotation.tab/bed genomesize *.bed"
        print "       Find the nearest annotation for given bed."
    else:
        # check file
        if '.tab' in sys.argv[1]:
            ftype='gene'
        else:
            ftype='bed'

        # initiation annotations.
        annos={}
        #for chrom in IO.genomeSize('hg19'):
        for chrom in Utils.genomeSize(sys.argv[2]):
            if ftype=='bed':
                annos[chrom]=BedList()
            else:
                annos[chrom]=GeneBedList()
        
        # read annotations.
        for anno in IO.BioReader(sys.argv[1],ftype=ftype):
            if annos.has_key(anno.chrom):
                annos[anno.chrom].append(anno)

        # sort
        for chrom in annos:
            annos[chrom].sort()

        # Find nearest annoations
        for item in IO.BioReader(sys.argv[3],ftype='bed'):
            if annos.has_key(item.chrom) and len(annos[item.chrom])>0:
                tanno=annos[item.chrom][annos[item.chrom].bisect(item)]
                olen=item.overlapLength(tanno)
                if olen==1:
                    annostr=tanno.id+':overlap'
                    olen=0
                elif olen>1:
                    annostr=tanno.id+':body'
                elif item.stop>tanno.start:
                    if tanno.strand=="+":
                        annostr=tanno.id+':upstream'
                    else:
                        annostr=tanno.id+':downstream'
                        olen=-olen
                else:
                    if tanno.strand=="+":
                        annostr=tanno.id+':downstream'
                        olen=-olen
                    else:
                        annostr=tanno.id+':upstream'
                print str(item)+"\t"+str(tanno)+"\t"+str(olen)+"\t"+annostr
    



#!/usr/bin/python
#Last-modified: 18 Mar 2014 03:54:08 PM

#         Module/Scripts Description
# 
# Copyright (c) 2008 Yunfei Wang <yfwang0405@gmail.com>
# 
# This code is free software; you can redistribute it and/or modify it
# under the terms of the BSD License (see the file COPYING included with
# the distribution).
# 
# @status:  experimental
# @version: 1.0.0
# @author:  Yunfei Wang
# @contact: yfwang0405@gmail.com

# ------------------------------------
# python modules
# ------------------------------------

import os,sys
import numpy
import string
import scipy
#from wLib.wBed import IO

# ------------------------------------
# constants
# ------------------------------------

# ------------------------------------
# Misc functions
# ------------------------------------

class test(object):
    ''' Statistics test. '''
    def ttest(a,b, axis=0, equal_var= True):
        '''
        Student test.
        
        Parameters:
            a,b: array_like
                The arrays must have the same shape, except in the dimension corresponding to axis (the first, by default).
            axis: int, optional
                Axis can equal None (ravel array first), or an integer (the axis over which to operate on a and b).
            equal_var: bool, optional
                If True (default), perform a standard independent 2 sample test that assumes equal population variances [R248]. If False, perform Welch’s t-test, which does not assume equal population variance [R249].
        
        Returns:
            t: float or array
                The calculated t-statistic.
            prob : float or array
                The two-tailed p-value.
        '''
        return scipy.stats.ttest_ind(a,b,axis,equal_var)
    ttest=staticmethod(ttest)
    def ranksums(a,b):
        '''
        Compute the Wilcoxon rank-sum statistic for two samples.
        
        The Wilcoxon rank-sum test tests the null hypothesis that two sets of measurements are drawn from the same distribution. The alternative hypothesis is that values in one sample are more likely to be larger than the values in the other sample.

        This test should be used to compare two samples from continuous distributions. It does not handle ties between measurements in x and y. For tie-handling and an optional continuity correction see scipy.stats.mannwhitneyu.

        Parameters:
            x,y : array_like
                The data from the two samples
        
        Returns :   
            z-statistic : float
                The test statistic under the large-sample approximation that the rank sum statistic is normally distributed
            p-value : float
                The two-sided p-value of the test
        '''
        return scipy.stats.ranksums(a,b)
    ranksums=staticmethod(ranksums)
    def wilcoxon(x, y=None, zero_method='wilcox', correction=False):
        '''
        Calculate the Wilcoxon signed-rank test.
        
        The Wilcoxon signed-rank test tests the null hypothesis that two related paired samples come from the same distribution. In particular, it tests whether the distribution of the differences x - y is symmetric about zero. It is a non-parametric version of the paired T-test.
        
        Parameters :    
            x : array_like
                The first set of measurements.
            y : array_like, optional
                The second set of measurements. If y is not given, then the x array is considered to be the differences between the two sets of measurements.
            zero_method : string, {“pratt”, “wilcox”, “zsplit”}, optional
                “pratt”:
                    Pratt treatment: includes zero-differences in the ranking process (more conservative)
                “wilcox”:
                    Wilcox treatment: discards all zero-differences
                “zsplit”:
                    Zero rank split: just like Pratt, but spliting the zero rank between positive and negative ones
            correction : bool, optional
                If True, apply continuity correction by adjusting the Wilcoxon rank statistic by 0.5 towards the mean value when computing the z-statistic. Default is False.
        
        Returns :   
            T : float
                The sum of the ranks of the differences above or below zero, whichever is smaller.
            p-value : float
                The two-sided p-value for the test.
        
        Note:
            Because the normal approximation is used for the calculations, the samples used should be large. A typical rule is to require that n > 20.
        '''
        return scipy.stats.wilcoxon(x,y,zero_method,correction)
    wilcoxon=staticmethod(wilcoxon)

class Utils(object):
    ''' Methods frequently used in statistics. '''
    def percentile(sortedv, p, interpolation=False):
        '''
        Return the item at the percentile of a sorted list.
        
        Parameters:
            sortedv: list or array
                sorted list or array
            p: float
                percentile of the list. Range from 0 to 1.
            interpolation: bool
                If false (default), return the nearest item. Otherwise, return the interpolated value of the two nearest items.
        
        Returns:
            item: not sure
                the item nearest to the pth percentile.
        '''
        if interpolation:
            s = len(sortedv)*p
            idxl = int(s)
            pl = s - idxl
            if pl != 0:
                idxl -= 1
            return sortedv[idxl]*(1-pl) + sortedv[idxl+1]*pl
        return sortedv[round(percent*len(sortedv))-1]
    percentile=staticmethod(percentile)
    def fivenum(v):
        '''
        Tukey's five number summary (minimum, lower-hinge, median, upper-hinge, maximum).
        
        Parameters:
            v: list or array
                a list of numbers. 
        
        Returns:
            fv: tuple
                (minimum, lower-hinge, median, upper-hinge, maximum). Hinge is defined as 1.5 IQR from median.
        '''
        l = len(v)-1
        nv = numpy.array(v)
        nv.sort()
        md = numpy.median(nv)
        q1 = percentile(nv, 0.25)
        q3 = percentile(nv, 0.75)
        whisker = 1.5*(q3 - q1)
        md = numpy.median(nv)
        return (nv[0], md-whisker, md, md+whisker, nv[1])
    fivenum=staticmethod(fivenum)
    
    def rank(x):
        ''' return the rank of x.'''
        array=numpy.array(x)
        temp = array.argsort()
        ranks = numpy.arange(len(array))[temp.argsort()]
        return ranks
    rank=staticmethod(rank)
    
    def tmean(x,lower=0,upper=1):
        ''' 
            trimmedmean of x. 
            Example: tmean (x,0.05,0.95)    
        '''
        array=numpy.array(x)
        if lower==0 and upper==1:
            return numpy.mean(array)
        else:
            array.sort()
            return numpy.mean(array[int(array.size*lower):int(array.size*upper)+1])
    tmean=staticmethod(tmean)

# ------------------------------------
# Classes
# ------------------------------------

# ------------------------------------
# Main
# ------------------------------------




#!/usr/bin/python
#Last-modified: 31 Jan 2013 11:01:07 AM

""" Module/Scripts Description

Copyright (c) 2008 Yunfei Wang <tszn1984@gmail.com>

This code is free software; you can redistribute it and/or modify it
under the terms of the BSD License (see the file COPYING included with
the distribution).

@status:  experimental
@version: 1.0.0
@author:  Yunfei Wang
@contact: tszn1984@gmail.com
"""

# ------------------------------------
# python modules
# ------------------------------------

import os,sys
import string
import argparse
from ngslib.wBed import Bed,Utils,IO
from random import randint

# ------------------------------------
# constants
# ------------------------------------

USERHOME=os.path.expanduser('~')

# ------------------------------------
# Misc functions
# ------------------------------------

def argParser():
    ''' Parse arguments. '''
    p=argparse.ArgumentParser(description="Generate random Bed regions from specified genome.")
    p._optionals.title = "Options"
    p.add_argument("-l","--length",dest='length',type=int,metavar="Length",required=True,help="Bed length.")
    p.add_argument("-n","--number",dest="number",type=int,metavar="Number",required=True,help="Number of Beds.")
    p.add_argument("-g","--genome",dest="genome",type=str,metavar="Genome",default=USERHOME+"/Data/hg19/hg19.sizes",help="Genome size file generated by faSize.")
    if len(sys.argv)==1:
        sys.exit(p.print_help())
    args = p.parse_args()
    return args

# ------------------------------------
# Classes
# ------------------------------------

# ------------------------------------
# Main
# ------------------------------------

if __name__=="__main__":
    # Get parameters
    args=argParser()
    #genome={}
    #for line in IO.ColumnReader(args.genome):
    #    genome[line[0]]=int(line[1])
    genome=Utils.genomeSize(args.genome)
    counts={}
    gsize=float(sum(genome.values()))
    # number of Beds in each chrom
    for chrom in genome:
        counts[chrom]=int(genome[chrom]/gsize*args.number+0.5)
    # check number
    diff=sum(counts.values())-args.number
    chroms=counts.keys()
    chroms.sort()
    if diff !=0:
        for i in diff>0 and range(diff) or range(diff,0):
            j=randint(1,len(counts))
            counts[chroms[j-1]]+= diff>0 and -1 or 1
    # generate Beds
    cnt=1
    for chrom in chroms:
        for i in xrange(counts[chrom]):
            pos=randint(0,genome[chrom]-args.length)
            print Bed([chrom,pos,pos+args.length,"bed_"+str(cnt)])
            cnt+=1





#!/usr/bin/python
#Last-modified: 04 Oct 2013 02:27:00 PM

#         Module/Scripts Description
# 
# Copyright (c) 2008 Yunfei Wang <Yunfei.Wang1@utdallas.edu>
# 
# This code is free software; you can redistribute it and/or modify it
# under the terms of the BSD License (see the file COPYING included with
# the distribution).
# 
# @status:  experimental
# @version: 1.0.0
# @author:  Yunfei Wang
# @contact: yfwang0405@gmail.com

# ------------------------------------
# python modules
# ------------------------------------

import os,sys
import string
import wWigIO

# ------------------------------------
# constants
# ------------------------------------

# ------------------------------------
# Misc functions
# ------------------------------------

# ------------------------------------
# Classes
# ------------------------------------

# ------------------------------------
# Main
# ------------------------------------

if __name__=="__main__":
    # get information from bigwig file
    wWigIO.open('test.bw')
    chroms = wWigIO.getChromSize('test.bw')
    wigs = wWigIO.getIntervals('test.bw', 'chr1', 10, 200)
    wWigIO.close('test.bw')
    print wigs

    # bigwig -> wig
    wWigIO.bigWigToWig('test.bw','test.wig')

    # write the chrom sizes into test.sizes
    with open('test.sizes','w') as fh:
        for chrom in chroms:
            print >>fh, chrom+"\t"+str(chroms[chrom])
    
    # wig -> bigwig
    wWigIO.wigToBigWig('test.wig','test.sizes','test2.bw')



#!/usr/bin/python
#Last-modified: 03 Oct 2013 09:52:14 PM

#         Module/Scripts Description
# 
# Copyright (c) 2008 Yunfei Wang <Yunfei.Wang1@utdallas.edu>
# 
# This code is free software; you can redistribute it and/or modify it
# under the terms of the BSD License (see the file COPYING included with
# the distribution).
# 
# @status:  experimental
# @version: 1.0.0
# @author:  Yunfei Wang
# @contact: yfwang0405@gmail.com

# ------------------------------------
# python modules
# ------------------------------------

import sys
import string
import wWigIO

# ------------------------------------
# constants
# ------------------------------------

# ------------------------------------
# Misc functions
# ------------------------------------

# ------------------------------------
# Classes
# ------------------------------------

class BigWigFile(object):
    '''  
    Fast reader of BigWig format file.
    Usage:
        Open file:
            fh=BigWigFile("test.bw")
        Get chromosome sizes:
            chroms=fh.chromSizes() # {key:value = chrom:size}
        Fetch regions:
            wigs=fh.fetch(chrom="chr1",start=100,stop=200)
            for start,stop,depth in wigs:
                #do some thing with wig
                print start,stop,depth
        Close file:
            fh.close()
    
    Parameters:
        chrom=None: return empty list.
        start=None: start at first position.
        stop=None:  stop at the end of chromosome.
    '''
    def __init__(self,fname):
        ''' Open BigWig file. '''
        self.fname=fname
        wWigIO.open(self.fname)
    def chromSizes(self):
        ''' Get chromosome sizes.'''
        chroms=wWigIO.getChromSize(self.fname)
        return chroms
    def fetch(self,**kwargs):
        ''' Fetch intervals in a given region. '''
        if kwargs.has_key('chrom'):
            wigs=wWigIO.getIntervals(self.fname,kwargs['chrom'],kwargs.get('start',0),kwargs.get('stop',0))
            if isinstance(wigs,basestring): # bad value
                raise ValueError("Couldn't get intervals.")
            return wigs
        raise ValueError("Chromosome not provided.")
        return
    def close(self):
        ''' Close BigWig file. '''
        wWigIO.close(self.fname)
    def __del__(self): 
        ''' Close BigWig file.  Avoid memory leaks.'''
        wWigIO.close(self.fname)

# ------------------------------------
# Main
# ------------------------------------

if __name__=="__main__":
    fh=BigWigFile("test.bw")
    chroms=fh.chromSizes() # {key:value = chrom:size}
    print chroms
    wigs=fh.fetch(chrom="chr1",start=100,stop=200)
    for start,stop,depth in wigs:
        #do some thing with wig
        print start,stop,depth




#!/usr/bin/python
#Last-modified: 29 Oct 2013 11:13:41 AM

""" Module/Scripts Description

Copyright (c) 2008 Yunfei Wang <tszn1984@gmail.com>

This code is free software; you can redistribute it and/or modify it
under the terms of the BSD License (see the file COPYING included with
the distribution).

@status:  experimental
@version: 1.0.0
@author:  Yunfei Wang
@contact: tszn1984@gmail.com
"""

# ------------------------------------
# python modules
# ------------------------------------

import re
import sys
import argparse
from ngslib import IO,FastaFile

# ------------------------------------
# constants
# ------------------------------------

# ------------------------------------
# Misc functions
# ------------------------------------

def argParser():
    ''' Parse arguments. '''
    p=argparse.ArgumentParser(description='Get a fragment from a Fasta file.',epilog='dependency ngslib')
    p._optionals.title = "Options"
    p.add_argument("-i","--input",dest='ifname',type=str,metavar="input.fa",required=True,help="Fasta file name.")
    p.add_argument("-r","--region",dest='region',type=str,metavar="chr1:100-200:+",required=False,help = 'Chromosome region. Leave it empty if not applicable, i.e. "chr1:100-:-".')
    p.add_argument("-c","--chrom",dest='chrom',type=str,metavar="chrom",required=False,help="chromosome name.")
    p.add_argument("-s","--start",dest='start',type=int,metavar="start",required=False,default=None,help="start coordinate. Default: begining of the chromosome.")
    p.add_argument("-e","--end",dest='end',type=int,metavar="end",required=False,default=None,help="end coordiante. Default: end of the chromosome.")
    p.add_argument("-t","--strand",dest='strand',type=str,metavar='strand',required=False,default="+",help='strand. Default: "+"')
    p.add_argument("-o","--output",dest='ofname',type=str, default="stdout", metavar="output.fa",required=False,help="Output file name. Default: stdout")
    if len(sys.argv)==1:
        sys.exit(p.print_help())
    args = p.parse_args()
    return args

# ------------------------------------
# Classes
# ------------------------------------

# ------------------------------------
# Main
# ------------------------------------

if __name__=="__main__":
    args = argParser()
    fh = IO.mopen(args.ofname, 'w')
    # Parse chromosome region
    if args.region:
        m = re.search("(\S+):(\S*)-(\S*):(\S*)",args.region)
        chrom, start, end, strand = m.groups()
    else:
        chrom = args.chrom
        start = args.start
        end   = args.end
        strand= args.strand
    if not chrom:
        raise ValueError("chromosome name is required.")
    start = int(start) if start else None
    end  = int(end) if end else None
    strand = strand if strand in ["+","-","."] else "+"
    # Get seq from Fasta file
    fa = FastaFile(args.ifname)
    seq = fa.getSeq(chrom,start,end,strand)
    fa.close()
    # Print result
    print >>fh, ">{0}:{1}-{2}:{3}".format(chrom, start and start or "", end and end or "", strand)
    print >>fh, seq.formatSeq()
    IO.mclose(fh)



#!/usr/Bin/python
#Last-modified: 14 Dec 2012 02:50:19 PM

""" Module/Scripts Description

Copyright (c) 2008 Yunfei Wang <tszn1984@gmail.com>

This code is free software; you can redistribute it and/or modify it
under the terms of the BSD License (see the file COPYING included with
the distribution).

@status:  experimental
@version: 1.0.0
@author:  Yunfei Wang
@contact: tszn1984@gmail.com
"""

# ------------------------------------
# python modules
# ------------------------------------

import os
import sys
import string
import argparse
import pysam
import numpy
import time
import ghmm
from ngslib import lprob
import wWigIO
from ngslib.wBed import Utils,IO,Peak
from ngslib.fdr import storey_qvalues


# ------------------------------------
# constants
# ------------------------------------

# ------------------------------------
# Misc functions
# ------------------------------------

def argParser():
    ''' Parse arguments. '''
    p=argparse.ArgumentParser(description='Convert BAM file to Wiggle file in specific Binsize.',epilog='dependency pysam')
    p._optionals.title = "Options"
    p.add_argument("-t",dest="treatments",type=str,metavar="chip.bam", required=True, nargs='*',help="The ChIP experiment BAM file(s). Histone marks supported are \"H3K4me1\", \"H3K4me3\", \"H3K27ac\", \"H3K27me3\", \"H3K36me3\". Histone mark names should be shown in file names.")
    p.add_argument("-c",dest="control",type=str,metavar="control",help="The ChIP control BAM file.")
    p.add_argument("-b",dest="Binsize",type=int,metavar='Binsize',default=50,help="Binsize  used to generate the Wiggle file. [default=50]")
    p.add_argument("-e",dest="extend", type=int,metavar='extend' ,default=150,help="Extend read length by strand. Set 0 if no extension. [default= 150bp]")
    p.add_argument("-n",dest="normedto",type=int,metavar="normedto",default=1000000,help="Reads normalization . Set 0 if no normalization. [default= 1M]")
    p.add_argument("-s",dest="split",action='store_true',default=False,help="\"-w\" required. Split wiggle into separate files by chromosomes.")
    p.add_argument("-o",dest="outputdir",type=str,metavar="ourputdir",default=".",help="Directory to put wiggle file(s). Default is current directory.")
    p.add_argument("-w",dest="wig",action='store_true',default=False,help="Save wiggle file. May use together with \"-s\".")
    if len(sys.argv)==1:
        sys.exit(p.print_help())
    args = p.parse_args()
    return args

def printTime():
    ''' Print time in fixed format. '''
    return time.strftime("#### %d %b %Y %H:%M:%S ####", time.gmtime())

class BamBins:
    ''' Bam file to Bins. '''
    def __init__(self):
        ''' Initiate BamBins. '''
        self.Binsize=50

    def bamToBins(samfile,Binsize=50,extend=150,normedto=1000000):
        ''' Extend BAM and pileup to Bins. '''
        Bins={}
        sam=pysam.Samfile(samfile,'rb')
        print >>sys.stderr, printTime(), "BAM->Bins ....."
        ratio= normedto==0 and 1.0 or float(normedto)/sam.mapped
        for tid,chrlen in enumerate(sam.lengths):
            Bins[sam.references[tid]]=numpy.zeros(chrlen/Binsize+extend/Binsize+1,'float')
        
        # Extension
        cnt=0
        start=stop=0
        for read in sam: # sam is 0-based
            if not read.is_unmapped:
                cnt+=1
                if cnt%10000==0:
                    print >>sys.stderr, "Processed ",cnt,"reads.      \r",
                if extend:
                    if read.is_reverse:
                        start, stop = max(0,read.pos+read.qlen-extend), read.pos+read.qlen-1
                    else:
                        start, stop = read.pos, read.pos+extend-1
                else:
                    start, stop = read.pos, read.pos+read.qlen-1
                # Add read to Bins
                chrom, startBin, stopBin = sam.references[read.tid], start/Binsize, stop/Binsize
                # For startBin: 
                # take base pos 6 (1 based) as an example, Binsize=50, readlen=60
                # pos 6-50 are in the startBin (50-6%50+1)
                # BAM file are 0 based, so bam.pos=5
                # So the bases in the startBin is Binsize-bam.pos%Binsize =
                Bins[chrom][startBin]-=start%Binsize 
                # For stopBin:
                # stop is pos 65 (1 based, 6+60-1). bam.stop is 64 (0 based)
                # pos 51-65  are in the stopBin ((65-1)%50+1=15) Note: it is different from 65%50=15.
                # So the bases in the stopBin is bam.stop%Binsize+1 = 64%50+1
                Bins[chrom][stopBin] +=stop%Binsize+1 
                Bins[chrom][startBin:stopBin]+=Binsize
        print >>sys.stderr, printTime(), "Prcessed ",cnt, "reads.     "
        
        
        # Normalization
        print >>sys.stderr, printTime(), "Normalization to", normedto, "reads. ratio=%-3.2f" % ratio
        for chrom in Bins:
            Bins[chrom]*=float(ratio)/Binsize
            if sam.lengths[sam.references.index(chrom)]%Binsize: # Set the last incomplete Bin to 0
                Bins[chrom][-1]=0
        sam.close()
    
        # time used
        print >>sys.stderr, printTime(), "Bam->Bins finished." 
        print >>sys.stderr
        return Bins
    bamToBins=staticmethod(bamToBins)
    def normalizeBins(treatBins,ctlBins=None):
        ''' Normalize Bins against control. '''
        if ctlBins:
            print >>sys.stderr, printTime(), "Normalized to control Bins."
            for chrom in treatBins:
                treatBins[chrom]-=ctlBins[chrom]
                treatBins[chrom]+=abs(treatBins[chrom])
                treatBins[chrom]/=2
        print >>sys.stderr
        return
    normalizeBins=staticmethod(normalizeBins)
    def BinsToWig(outputdir,samfile,Bins,chrsplit=False):
        ''' Print to wig files. '''
        # Get bname prefix
        bname=os.path.basename(samfile)
        bname=outputdir+"/"+os.path.splitext(bname)[0]
        
        # Binsize
        Binsize=self.Binsize
        
        # Open wiggle file
        if not chrsplit:
            fh=open(bname+".wig",'w')
            print >>fh, "track type=wiggle_0 name="+os.path.basename(bname)
        
        # Print wiggle
        for chrom in Bins:
            print >>sys.stderr, printTime(), "Bins->Wiggle ", chrom
            
            # Open Wiggle file
            if chrsplit:
                fh=open(bname+"_"+chrom+".wig",'w')
                print >>fh, "track type=wiggle_0 name="+os.path.basename(bname)+"_"+chrom
            
            # Print wiggle lines
            print >>fh, "variableStep chrom="+chrom+" span="+str(Binsize)
            for i,d in enumerate(Bins[chrom]):
                depth=float("%.2f" % d)
                if depth>0:
                    print >>fh, "%d\t%-3.2f" % (i*Binsize+1,depth)
            
            # Close file handle
            if chrsplit:
                fh.close()
        
        # Close file handle
        if not chrsplit:
            fh.close()
    
        print >>sys.stderr, printTime(), "Bins->Wiggle finished."
        print >>sys.stderr
        return
    BinsToWig=staticmethod(BinsToWig)
    def BinsFromBigWig(bwfile,binsize):
        ''' Read bins from BigWig file.'''
        Bins={}
        chroms=Utils.getBigWigChroms(bwfile)
        for chrom in chroms:
            Bins[chrom]=numpy.zeros(chroms[chrom]/binsize)
            for start,stop,val in IO.BigWigReader(bwfile,chrom):
                Bins[chrom][start/binsize]=val
        return Bins
    BinsFromBigWig=staticmethod(BinsFromBigWig)
    def callPeakFromBins(Bins,pvalue=1e-5):
        ''' Call peak with poisson distribution. Return the hmmStates, lamda and threshold for each chromosome.'''
        print >>sys.stderr, printTime(), "Call peak for each chromosome."
        hmmStates={}
        lams={}
        for chrom in Bins:
            # All bin values are multiplied by 100, because the threshold is usually integers.
            lam = sum(Bins[chrom])/len(Bins[chrom])*100
            # find the optimal threshold by binary search
            lthre=0.0
            hthre=max(Bins[chrom])*100.0  # 
            while hthre-lthre>0.01:
                pval = lprob.poisson_cdf((hthre+lthre)/2,lam,False)
                if pval > pvalue:
                    lthre=(hthre+lthre)/2
                else:
                    hthre=(hthre+lthre)/2
            # Print threshold
            print >>sys.stderr, printTime(), chrom, "lambda=",lam,"threshold=",hthre
            # convert to binary value
            hmmStates[chrom]=(numpy.greater_equal(Bins[chrom],hthre/100)-0).tolist()
            lams[chrom]=lam
        print >>sys.stderr, printTime(), "Call peak finished."
        print >>sys.stderr
        return (hmmStates,lams)
    callPeakFromBins=staticmethod(callPeakFromBins)
    def trainHMM(hmmState):
        ''' Train HMM with the given chromosome. '''
        print >>sys.stderr, printTime(), "Train HMM with one chromosome."
        T=[[0.9,0.1],[0.1,0.9]]
        e1=[0.1,0.9]
        e0=[0.9,0.1]
        E=[e0,e1]
        pi=[0.9,0.1] # initial 10% are peak?
        sigma=ghmm.IntegerRange(0,2) # 0, 1
        m = ghmm.HMMFromMatrices(sigma,ghmm.DiscreteDistribution(sigma),T,E,pi)
        m.baumWelch(ghmm.EmissionSequence(sigma,hmmState))
        print >>sys.stderr, printTime(), "Train HMM finished."
        print >>sys.stderr
        return m
    trainHMM=staticmethod(trainHMM)
    def decodeHMM(m,hmmStates):
        ''' Decode HMM. '''
        print >>sys.stderr, printTime(), "Decode HMM for each chromosome."
        sigma=ghmm.IntegerRange(0,2)
        for chrom in hmmStates:
            print >>sys.stderr, printTime(), chrom
            state, score = m.viterbi(ghmm.EmissionSequence(sigma,hmmStates[chrom]))
            hmmStates[chrom]=state
        print >>sys.stderr, printTime(), "Decode HMM finished."
        print >>sys.stderr
    decodeHMM=staticmethod(decodeHMM)
    def parseHMM(hmmStates,Binsize):
        ''' Parse HMM to peaks. '''
        for chrom in hmmStates:
            print >>sys.stderr, printTime(), chrom
            flag=False
            for i,s in enumerate(hmmStates[chrom]):
                if s==1:  # current =1
                    if flag: # last =1 
                        stop+=1
                    else:    # last =0 ,start of peak
                        start=i
                        stop=i
                        flag=True
                else:    # curent =0
                    if flag:  # last =1, end of peak
                        yield (chrom,start*Binsize,(stop+1)*Binsize)
                        flag=False
            if flag: # end of chromosome
                yield (chrom,start*Binsize,(stop+1)*Binsize)
        return
    parseHMM=staticmethod(parseHMM)
    def HMMToPeaks(hmmStates,treatbins,Binsize):
        ''' Parse HMM result to Peaks. '''
        # parse HMM and generate peak files.
        print >>sys.stderr, printTime(), "Parse Peaks from HMM result."

        peaks=[]
        pval=[]
        cnt=1
        for chrom,start,stop in BamBins.parseHMM(hmmStates,Binsize):
            tpeak=Peak([chrom,start,stop,histmark+"_"+str(cnt)])
            bins=treatBins[chrom][start/Binsize:stop/Binsize]
            tpeak.score=max(bins)
            tpeak.signalvalue=numpy.mean(bins)
            tpeak.pvalue = -10*numpy.log10(lprob.poisson_cdf(tpeak.score*100,lams[chrom],False))
            tpeak.peak=bins.argmax()*Binsize+Binsize/2
            # append to peaks
            peaks.append(tpeak)
            #pval.append(tpeak.pvalue)
            cnt+=1
        #qval=storey_qvalues(pval)
        #for i in xrange(len(peaks)):
            #peaks[i].pvalue=-10*numpy.log10(pval[i])
            #peaks[i].qvalue=-10*numpy.log10(qval[i])
        print >>sys.stderr, printTime(), "Parse Peaks finished."
        print >>sys.stderr
        return peaks
    HMMToPeaks=staticmethod(HMMToPeaks)
    

# ------------------------------------
# Classes
# ------------------------------------

# ------------------------------------
# Main
# ------------------------------------

if __name__=="__main__":
    # Get parameters
    args=argParser()
    Binsize=args.Binsize
    extend=args.extend
    normedto=args.normedto
    chromsplit=args.split
    outputdir=args.outputdir
    towig=args.wig
    if not os.path.exists(outputdir):
        os.makedirs(outputdir)
    histmarks="H3K4me3,H3K4me1,H3K27ac,H3K27me3,H3K36me3".split(",")
    treatments={}
    for f in args.treatments:
        for h in histmarks:
            if h.upper() in f.upper():
                treatments[h]=f
    
    # Print parameters
    print >>sys.stderr, printTime(), "Parameters:"
    print >>sys.stderr, printTime(), "    Binsize=", Binsize,"bp"
    print >>sys.stderr, printTime(), extend   and "    extend to %d bp" % extend or "    No extension"
    print >>sys.stderr, printTime(), normedto and "    normalized to %d reads" % normedto or "    No normalization"
    if chromsplit:
        print >>sys.stderr, printTime(), "    Split wiggle by chromosomes."
    print >>sys.stderr, printTime(), outputdir=="." and "    output in current dir" or "    output in "+outputdir
    print >>sys.stderr
    
    # read control
    ctlBins=None
    if args.control:
        print >>sys.stderr, printTime(), "Reading control:",args.control
        if args.control.endswith("bam"):
            print >>sys.stderr, printTime(), "File format is BAM."
            ctlBins=BamBins.bamToBins(args.control,Binsize,extend,normedto)
        elif args.control.endswith("bw") or args.control.endswith("bigwig"):
            print >>sys.stderr, printTime(), "File format is BigWig."
            ctlBins=BamBins.BinsFromBigWig(args.control,Binsize)
        else:
            sys.exit(printTime()+ "ERROR: "+args.control+" file format not suported. Options are *.bam, *.bw or *.bigwig.")            
    
    # read treatment
    Peaks={}
    for histmark in treatments:
        print >>sys.stderr, printTime(), "Reading treatment:",treatments[histmark]
        # BAM to Bins
        if treatments[histmark].endswith("bam"):
            print >>sys.stderr, printTime(), "File format is BAM."
            treatBins=BamBins.bamToBins(treatments[histmark],Binsize,extend,normedto)
        elif treatments[histmark].endswith("bw") or treatments[histmark].endswith("bigwig"):
            print >>sys.stderr, printTime(), "File format is BigWig."
            treatBins=BamBins.BinsFromBigWig(treatments[histmark],Binsize)
        else:
            sys.exit(printTime()+ "ERROR"+treatments[histmark]+" file format not suported. Options are *.bam, *.bw or *.bigwig.")
        
        # normalize Bins
        BamBins.normalizeBins(treatBins,ctlBins)
        
        # Wiggle output
        if towig:
            BamBins.BinsToWig(outputdir,treatments[histmark],treatBins,Binsize,chromsplit)

        # call peak
        hmmStates,lams= BamBins.callPeakFromBins(treatBins)

        # train HMM and decode HMM. hmmStates will be updated.
        m=BamBins.trainHMM(hmmStates['chr1'])
        BamBins.decodeHMM(m,hmmStates)

        # parse HMM and generate peak files. *.bam -> *.peaks
        Peaks[histmark] = BamBins.HMMToPeaks(hmmStates,treatBins,Binsize)
        print >>sys.stderr, "Write Peaks to file:",treatments[histmark].split(".")[0]+".peaks"
        fh=open(treatments[histmark].split(".")[0]+".peaks",'w')
        for tpeak in Peaks[histmark]:
            print >>fh, tpeak
        fh.close()
        print >>sys.stderr, "Write Peaks finished."
        print >>sys.stderr 
    

    





#!/usr/bin/python
#Last-modified: 07 Mar 2013 01:37:19 PM

#         Module/Scripts Description
# 
# Copyright (c) 2008 Yunfei Wang <Yunfei.Wang1@utdallas.edu>
# 
# This code is free software; you can redistribute it and/or modify it
# under the terms of the BSD License (see the file COPYING included with
# the distribution).
# 
# @status:  experimental
# @version: 1.0.0
# @author:  Yunfei Wang
# @contact: Yunfei.Wang1@utdallas.edu

# ------------------------------------
# python modules
# ------------------------------------

import sys
import string
from ngslib.wBed import IO

# ------------------------------------
# constants
# ------------------------------------

# ------------------------------------
# Misc functions
# ------------------------------------

# ------------------------------------
# Classes
# ------------------------------------

# ------------------------------------
# Main
# ------------------------------------

if __name__=="__main__":
    if len(sys.argv)==1:
        sys.exit("Example:"+sys.argv[0]+" *.bed *.bw ")
    for item in IO.ColumnReader(sys.argv[1],ftype='bed'):
        wigs=item.getWig(sys.argv[2])
        if len(wigs)>0:
            w=[wigs[i][2] for i in xrange(len(wigs))]
            item.score=max(w)
            bestpos=w.index(item.score)
        print item+"\t"+str(bestpos)




#!/usr/bin/python
#Last-modified: 20 Apr 2013 05:56:53 PM

#         Module/Scripts Description
# 
# Copyright (c) 2008 Yunfei Wang <Yunfei.Wang1@utdallas.edu>
# 
# This code is free software; you can redistribute it and/or modify it
# under the terms of the BSD License (see the file COPYING included with
# the distribution).
# 
# @status:  experimental
# @version: 1.0.0
# @author:  Yunfei Wang
# @contact: Yunfei.Wang1@utdallas.edu

# ------------------------------------
# python modules
# ------------------------------------

import sys
import string
import pysam

# ------------------------------------
# constants
# ------------------------------------

# ------------------------------------
# Misc functions
# ------------------------------------

# ------------------------------------
# Classes
# ------------------------------------

# ------------------------------------
# Main
# ------------------------------------

if __name__=="__main__":
    if len(sys.argv)==1:
        sys.exit("Example:"+sys.argv[0]+" *.bam/sam >*.bed ")
    sam=pysam.Samfile(sys.argv[1], sys.argv[1].endswith("bam") and 'rb' or 'r')
    for read in sam:
        if not read.is_unmapped:
            print "%s\t%d\t%d\t%s\t%d\t%s" %(sam.references[read.tid],read.pos,read.aend,read.qname,read.mapq,read.is_reverse and "-" or "+")
    sam.close()




#!/usr/bin/python
#Last-modified: 18 Mar 2014 03:55:29 PM

""" Module/Scripts Description

Copyright (c) 2008 Yunfei Wang <yfwang0405@gmail.com>

This code is free software; you can redistribute it and/or modify it
under the terms of the BSD License (see the file COPYING included with
the distribution).

@status:  experimental
@version: 1.0.0
@author:  Yunfei Wang
@contact: yfwang0405@gmail.com
"""

# ------------------------------------
# python modules
# ------------------------------------

import os
import sys
import string
import argparse
import pysam
import numpy
import time

# ------------------------------------
# constants
# ------------------------------------

# ------------------------------------
# Misc functions
# ------------------------------------

def argParser():
    ''' Parse arguments. '''
    p=argparse.ArgumentParser(description='Convert BAM file to Wiggle file in specific binsize. Reads are counted multiple times if not uniquely mapped. Contact Yunfei Wang to report any bugs (yfwang0405@gmail.com).',epilog='dependency pysam')
    p._optionals.title = "Options"
    p.add_argument("-t",dest="treatments",type=str,metavar="chip.bam", required=True, nargs='+',help="The ChIP experiment BAM file(s).")
    p.add_argument("-c",dest="control",type=str,metavar="control",default=None,help="The ChIP control BAM file.")
    p.add_argument("-b",dest="binsize",type=int,metavar='binsize',default=50,help="Binsize  used to generate the Wiggle file. [default=50]")
    p.add_argument("-e",dest="extend", type=int,metavar='extend' ,default=150,help="Extend read length by strand. Set \"0\" if no extension. [default= 150bp]")
    p.add_argument("-n",dest="normedto",type=int,metavar="normedto",default=10,help="Reads normalization . Set \"0\" if no normalization. Set \"10\" if want to normalize to 10 M reads. [default= 10]")
    p.add_argument("-R","--RNASeq",dest="RNASeq",action='store_true',default=False,help="Input is RNA Seq reads. No extension. [default= False].")
    p.add_argument("-P","--Paired",dest="Pairend",action='store_true',default=False,help="Input is paired end reads. No extension. [default= False].")
    p.add_argument("-s","--split",dest="split",action='store_true',default=False,help="Split wiggle into separate files by chromosomes. [default= False].")
    p.add_argument("-f","--forcestrand",dest="forcestrand",action='store_true',default=False,help="Force strand. [default= False].")
    p.add_argument("-p",dest="prefix",type=str,metavar="prefix",default="",help="Prefix to the output file name in format: Prefix_BamFileName_CtlBamFileName.wig. [default= \"\"]")
    p.add_argument("-o",dest="outputdir",type=str,metavar="ourputdir",default=".",help="Directory to put wiggle file(s). [default= \".\"].")
    if len(sys.argv)==1:
        sys.exit(p.print_help())
    args = p.parse_args()
    return args

def Merge(beds):
    newbeds=[]
    newbeds.append(list(beds[0]))
    for start,stop in beds[1:]:
        if newbeds[-1][0]<= start <=newbeds[-1][1]:
            newbeds[-1][1]=max(newbeds[-1][1],stop)
        else:
            newbeds.append([start,stop])
    return newbeds

def bamToBins(bname,binsize=50,extend=150,normedto=10000000,paired=False,forcestrand=False):
    ''' Extend BAM and pileup to bins. '''
    # strand check
    if forcestrand:
        bins={"+":{},"-":{}}
    else:
        bins={".":{}}
    # Open bamfile
    sam=pysam.Samfile(bname,'rb')
    print >>sys.stderr, "BAM->bins ....."
    starttime=time.clock()
    ratio= normedto==0 and 1.0 or float(normedto)/sam.mapped
    # Bins initiation
    for tid,chrlen in enumerate(sam.lengths):
        for strand in bins:
            bins[strand][sam.references[tid]]=numpy.zeros(chrlen/binsize+extend/binsize+2,'float') # the number of bins > chromlen/binsize to simplify computation
    
    # For single end: Extension
    if not paired:
        cnt=0
        start=stop=0
        strand="."
        for read in sam: # sam is 0-based
            if not read.is_unmapped:
                cnt+=1
                if extend:
                    if read.is_reverse:
                        start, stop = max(0,read.pos+read.qlen-extend), read.pos+read.qlen-1
                    else:
                        start, stop = read.pos, read.pos+extend-1
                else:
                    start, stop = read.pos, read.pos+read.qlen-1
                # Add read to bins
                chrom, startbin, stopbin= sam.references[read.tid], start/binsize, stop/binsize
                strand = forcestrand and (read.is_reverse and "-" or "+") or "."
                # For startbin: 
                # take base pos 6 (1 based) as an example, binsize=50, readlen=60
                # pos 6-50 are in the startbin (50-6%50+1)
                # BAM file are 0 based, so bam.pos=5
                # So the bases in the startbin is binsize-bam.pos%binsize =
                bins[strand][chrom][startbin]-=start%binsize 
                # For stopbin:
                # stop is pos 65 (1 based, 6+60-1). bam.stop is 64 (0 based)
                # pos 51-65  are in the stopbin ((65-1)%50+1=15) Note: it is different from 65%50=15.
                # So the bases in the stopbin is bam.stop%binsize+1 = 64%50+1
                bins[strand][chrom][stopbin] +=stop%binsize+1 
                bins[strand][chrom][startbin:stopbin]+=binsize
        print >>sys.stderr, "Prcessed ",cnt, "reads.     "
    # For paired end: consider overlap
    else:
        cnt=0
        reads={}
        for read in sam:
            cnt+=1
            if read.is_unmapped==False and read.mate_is_unmapped==False and read.tid==read.rnext: # both mapped and in the same chrom
                if reads.has_key(read.qname): # has pair in reads
                    # find the mate pair
                    materead=None
                    for idx,tread in enumerate(reads[read.qname]):
                        if tread.tid == read.tid and tread.pos == read.pnext: # same chrom, mated
                            materead=tread
                            mateidx=idx
                    if materead: # if mated
                        # check the strand and pair
#                       if (read.is_read1 and materead.is_read2) or (read.is_read2 and materead.is_read1):
#                           print "Paired"
#                       else:
#                           print "not paired"
#                       if (read.is_reverse!=materead.is_reverse):
#                           print "complementary"
#                       else:
#                           print "same strand"

                        strand= forcestrand and ((read.is_read1 != read.is_reverse) and "+" or "-") or "."
                        #processing CIGAR  17M204733N83M=[(0, 17), (3, 204733), (0, 83)]
                        beds=[]
                        # one read
                        start=read.pos
                        for mtype,mlen in read.cigar:
                            if mtype==0: # match
                                beds.append((start,start+mlen-1))
                            elif mtype==1: # insertion
                                start-=mlen
                            start+=mlen
                        # the other read
                        start=materead.pos
                        for mtype,mlen in materead.cigar:
                            if mtype==0: # match
                                beds.append((start,start+mlen-1))
                            elif mtype==1: # insertion
                                start-=mlen
                            start+=mlen
                        # Add bed to bins
                        for start,stop in Merge(beds): # the beds are merged to avoid overlap between paired ends.
                            chrom, startbin, stopbin= sam.references[read.tid], start/binsize, stop/binsize
                            try:
                                bins[strand][chrom][startbin]-=start%binsize
                                bins[strand][chrom][stopbin] +=stop%binsize+1
                                bins[strand][chrom][startbin:stopbin]+=binsize
                            except:
                                print read
                                print reads[read.qname]
                                print read.cigar,",",reads[read.qname].cigar
                                print chrom, start, stop, startbin, stopbin
                                sys.exit()
                        # Delete read from reads
                        del reads[read.qname][mateidx]
                        if len(reads[read.qname])==0:
                            del reads[read.qname]
                    else: # not mate
                        reads[read.qname].append(read)
                else:
                    reads[read.qname]=[read]
        print >>sys.stderr, "Prcessed ",cnt, "reads.     "

    # Normalization
    print >>sys.stderr, "Normalization ratio=%-3.2f" % ratio
    for strand in bins:
        for chrom in bins[strand]:
            bins[strand][chrom]*=float(ratio)/binsize
            if sam.lengths[sam.references.index(chrom)]%binsize: # Set the last incomplete bin to 0
                bins[strand][chrom][-1]=0
    sam.close()

    # time used
    endtime=time.clock()
    print >>sys.stderr, "Time used: %-3.2f secs." % (endtime-starttime)
    return bins

def binsToWig(prefix,outputdir,samfile,bins,control,ctlbins=None,chrsplit=False):
    ''' Print to wig files. '''
    starttime=time.clock()
    # Get bname prefix
    bname=os.path.basename(samfile)
    bname=os.path.splitext(bname)[0]
    if prefix!="":
        bname=prefix+"_"+bname
    bname=outputdir+"/"+bname
    if control:
        bname += "_"+os.path.splitext(os.path.basename(control))[0]
    
    # chroms
    chroms={}
    sam=pysam.Samfile(samfile,'rb')
    for tid,length in enumerate(sam.lengths):
        chroms[sam.references[tid]]=length
    sam.close()

    # binsize
    chrom=chroms.keys()[0]
    strand=bins.keys()[0]
    binsize=chroms[chrom]/len(bins[strand][chrom])+1
    
    # strand
    for strand in bins:
        if strand!=".":
            suffix= strand=="+" and "_plus.wig" or "_minus.wig"
        else:
            suffix=".wig"
        # Open wiggle file
        if not chrsplit:
            fh=open(bname+suffix,'w')
            print >>fh, "track type=wiggle_0 name="+os.path.basename(bname)
        
        # Print wiggle
        for chrom in sorted(chroms.keys()):
            length=chroms[chrom]
            print >>sys.stderr, "Bins->Wiggle ", chrom, strand, '  \r',
            
            # Open Wiggle file
            if chrsplit:
                fh=open(bname+"_"+chrom+suffix,'w')
                print >>fh, "track type=wiggle_0 name="+os.path.basename(bname)+"_"+chrom
            
            # normalized to control
            if ctlbins and ctlbins[strand].has_key(chrom):
                bins[strand][chrom]-=ctlbins[strand][chrom]
                bins[strand][chrom]+=abs(bins[strand][chrom])
                bins[strand][chrom]/=2
            
            # Print wiggle lines
            print >>fh, "variableStep chrom="+chrom+" span="+str(binsize)
            bins[strand][chrom][length/binsize:] = 0 # ignore the bins >length/binsize
            for i in numpy.nonzero(bins[strand][chrom])[0]:
                print >>fh, "%d\t%-3.2f" % (i*binsize+1, bins[strand][chrom][i])
            
            # Close file handle
            if chrsplit:
                fh.close()
    
    # Close file handle
    if not chrsplit:
        fh.close()

    print >>sys.stderr
    endtime=time.clock()
    print >>sys.stderr, "Time used: %-3.2f secs." % (endtime-starttime)
    return

# ------------------------------------
# Classes
# ------------------------------------

# ------------------------------------
# Main
# ------------------------------------

if __name__=="__main__":
    # Get parameters
    args=argParser()
    binsize=args.binsize
    extend=args.extend
    normedto=args.normedto*1000000
    chromsplit=args.split
    treatments=args.treatments
    outputdir=args.outputdir
    prefix=args.prefix
    forcestrand=args.forcestrand
    isPaired=args.Pairend
    isRNASeq = args.RNASeq
    if isPaired or isRNASeq: # No extension if RNA Seq
        extend=0
    if not os.path.exists(outputdir):
        os.makedirs(outputdir)
    
    # Print parameters
    print >>sys.stderr, "Parameters:"
    print >>sys.stderr, "\tbinsize=", binsize,"bp"
    print >>sys.stderr, isPaired and "\tPaired end reads." or "\tSingle end reads."
    print >>sys.stderr, extend   and "\textend to %d bp" % extend or "\tNo extension"
    print >>sys.stderr, normedto and "\tnormalized to %dM reads" % args.normedto or "\tNo normalization"
    if isRNASeq:
        print >>sys.stderr, "\tRNA Seq reads."
    print >>sys.stderr, forcestrand and "\tForce strand." or "\tDon't force strand."
    if chromsplit:
        print >>sys.stderr, "\tSplit wiggle by chromosomes."
    print >>sys.stderr, outputdir=="." and "\toutput in current dir" or "\toutput in "+outputdir
    print >>sys.stderr
    
    # read control
    ctlbins=None
    if args.control:
        print >>sys.stderr, "Reading control:",args.control
        ctlbins=bamToBins(args.control,binsize,extend,normedto,isPaired,forcestrand)
    
    # read treatment
    for samfile in treatments:
        print >>sys.stderr, "Reading treatment:",samfile
        # Bam to Bins
        treatbins=bamToBins(samfile,binsize,extend,normedto,isPaired,forcestrand)
        # Wiggle output
        binsToWig(prefix,outputdir,samfile,treatbins,args.control,ctlbins,chromsplit)

        





#!/usr/bin/python
#Last-modified: 29 Oct 2013 11:16:36 AM

""" Module/Scripts Description

Copyright (c) 2008 Yunfei Wang <tszn1984@gmail.com>

This code is free software; you can redistribute it and/or modify it
under the terms of the BSD License (see the file COPYING included with
the distribution).

@status:  experimental
@version: 1.0.0
@author:  Yunfei Wang
@contact: tszn1984@gmail.com
"""

# ------------------------------------
# python modules
# ------------------------------------

import sys
import argparse
from ngslib import IO,FastaFile

# ------------------------------------
# constants
# ------------------------------------

# ------------------------------------
# Misc functions
# ------------------------------------

def argParser():
    ''' Parse arguments. '''
    p=argparse.ArgumentParser(description='Get sequences by a list of names.',epilog='dependency ngslib')
    p._optionals.title = "Options"
    p.add_argument("-i","--input",dest='ifname',type=str,metavar="input.fa",required=True,help='Fasta file name.')
    p.add_argument("-n","--names",dest='names',type=str,metavar="names.lst",required=True,help='A file with sequence names. Can be "stdin".')
    p.add_argument("-o","--output",dest='ofname',type=str, default="stdout", metavar="output.fa",required=False,help='Output file name. Default is "stdout".')
    if len(sys.argv)==1:
        sys.exit(p.print_help())
    args = p.parse_args()
    return args

# ------------------------------------
# Classes
# ------------------------------------

# ------------------------------------
# Main
# ------------------------------------

if __name__=="__main__":
    args = argParser()
    fh = IO.mopen(args.ofname, 'w')
    # Fasta file
    fa = FastaFile(args.ifname)
    f = IO.mopen(args.names)
    for faid in f:
        faid=faid.rstrip()
        seq=fa.getSeq(faid)
        print >> fh, ">"+faid
        print >> fh, seq.formatSeq()
    fa.close()
    IO.mclose(f)
    IO.mclose(fh)

